#!/usr/bin/env python3

import os
import sys
import argparse
import re
import subprocess
import logging
import json

from cvehound import CVEhound
from cvehound.util import *
from cvehound.exception import UnsupportedVersion
from cvehound.cwe import CWE

def main(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(
        prog='cvehound',
        description='A tool to check linux kernel sources dump for known CVEs',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument('--version', action='version', version=get_cvehound_version())
    parser.add_argument('--all-files', action='store_true',
                        help="don't use files hint from cocci rules")
    parser.add_argument('--cve', '-c', nargs='+', default='all',
                        help='list of cve identifiers')
    parser.add_argument('--cwe', nargs='+', default=[], type=int,
                        help='check only for CWE-ids')
    parser.add_argument('--files', nargs='+', default=[],
                        help='check only files (e.g. drivers/block/floppy.c arch/x86)')
    parser.add_argument('--kernel', '-k', type=dir_path, required=True,
                        help='linux kernel sources dir')
    parser.add_argument('--config', nargs='?', type=file_path, const='-',
                        help='check kernel config')
    parser.add_argument('--report', nargs='?', const='report.json',
                        help='output report with found CVEs')
    parser.add_argument('--report-strict', action='store_true',
                        help='include in report only CVEs included in .config')
    parser.add_argument('-v', '--verbose', action='count', default=0,
                        help='increase output verbosity')
    cmdargs = parser.parse_args()

    if not tool_exists('spatch'):
        print('Please, install coccinelle')
        sys.exit(1)

    if cmdargs.config == '-':
        config = os.path.join(cmdargs.kernel, '.config')
        if os.path.isfile(config):
            cmdargs.config = config
    if cmdargs.config and cmdargs.verbose == 0:
        cmdargs.verbose = 1

    if cmdargs.report_strict:
        if not cmdargs.report:
            cmdargs.report = 'report.json'
        if not cmdargs.config:
            print('Please, use --config with --report-strict')
            sys.exit(1)

    loglevel = logging.WARNING
    if cmdargs.verbose > 1:
        loglevel = logging.DEBUG
    elif cmdargs.verbose > 0:
        loglevel = logging.INFO
    logging.basicConfig(level=loglevel, format='%(message)s')

    config_info = {}
    if cmdargs.config != '-':
        config_info = get_config_data(cmdargs.config)

    hound = CVEhound(cmdargs.kernel, cmdargs.config, config_info.get('arch', 'x86'))

    known_cves = hound.get_cves()
    if cmdargs.cve == 'all':
        cmdargs.cve = known_cves
    else:
        cve_id = re.compile(r'^CVE-\d{4}-\d{4,7}$')
        for i, cve in enumerate(cmdargs.cve):
            if not cve.startswith('CVE-'):
                cve = 'CVE-' + cve
                cmdargs.cve[i] = cve
            if not cve_id.match(cve):
                print('Wrong CVE-ID:', cve, file=sys.stderr)
                sys.exit(1)
            if cve not in known_cves:
                print('Unknown CVE:', cve, file=sys.stderr)
                sys.exit(1)

    filter_cwes = frozenset(cmdargs.cwe)

    if cmdargs.all_files and not cmdargs.files:
        print('--files filter and --all-files are not compatible', file=sys.stderr)
        sys.exit(1)
    for f in cmdargs.files:
        path = re.compile(r'^[a-zA-Z-./0-9]+$')
        if not path.match(f):
            print('Wrong file filter:', f, file=sys.stderr)
            sys.exit(1)

    cves = []
    for cve in cmdargs.cve:
        if cmdargs.cwe:
            rule_cwe_desc = hound.get_cve_cwe(cve)
            if not rule_cwe_desc:
                continue
            rule_cwes = frozenset(CWE[rule_cwe_desc])
            if not (rule_cwes & filter_cwes):
                continue
        if cmdargs.files:
            found = False
            for rulefile in hound.get_rule_files(cve):
                for filterdir in cmdargs.files:
                    if rulefile.startswith(filterdir):
                        found = True
            if not found:
                continue
        cves.append(cve)
    cmdargs.cve = cves

    report = { 'args': {}, 'kernel': {}, 'config': {}, 'tools': {}, 'results': {}}
    report['args']['cve'] = cmdargs.cve
    report['args']['kernel'] = cmdargs.kernel
    report['args']['config'] = cmdargs.config
    report['args']['only_cwe'] = cmdargs.cwe
    report['args']['only_files'] = cmdargs.files
    report['args']['all_files'] = cmdargs.all_files
    report['kernel'] = get_kernel_version(cmdargs.kernel)
    if cmdargs.config != '-':
        report['config'] = config_info
    report['tools']['cvehound'] = get_cvehound_version()
    report['tools']['spatch'] = '.'.join(list(str(get_spatch_version())))
    for cve in cmdargs.cve:
        try:
            result = hound.check_cve(cve, cmdargs.all_files)
            if result and (not cmdargs.report_strict or result['config']['affected']):
                report['results'][cve] = result
        except subprocess.CalledProcessError as e:
            logging.error('Failed to run: ' + ' '.join(e.cmd))
        except UnsupportedVersion as err:
            logging.error('Skipping: ' + err.cve + ' requires spatch >= ' + err.rule_version)

    if cmdargs.report:
        with open(cmdargs.report, 'wt', encoding='utf-8') as fh:
            json.dump(report, fh, indent=4, sort_keys=True)
        print('Report saved to:', cmdargs.report)

if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3

import pkg_resources
import pytest
import os

from cvehound.exception import UnsupportedVersion

@pytest.mark.slow
def test_between_fixes_fix(hound, repo, cve):
    fix = hound.get_rule_fix(cve)
    fixes = hound.get_rule_fixes(cve)

    repo.git.checkout(fixes)
    tags = repo.git.rev_list('--no-merges', '--simplify-by-decoration',
                             '--ancestry-path', fixes + '..' + fix)
    for tag in tags.split():
        repo.git.checkout(tag)
        try:
            assert hound.check_cve(cve), cve + ' fails to detect on ' + tag
        except UnsupportedVersion:
            pytest.skip('Unsupported spatch version')



#!/usr/bin/env python3

import os
import sys
import argparse
import re
import subprocess
import logging
from subprocess import PIPE
import pkg_resources
import collections
from datetime import datetime
from sympy.logic import simplify_logic
from sympy import symbols
from cvehound.cpu import CPU
from cvehound.exception import UnsupportedVersion
from cvehound.util import get_spatch_version, get_all_cves, get_cves_metadata
from cvehound.kbuild import KbuildParser
from cvehound.config import Config

__VERSION__ = '1.0.1'


class CVEhound:

    def __init__(self, kernel, config=None, arch='x86'):
        self.kernel = kernel
        self.metadata = get_cves_metadata()
        self.cocci_job = str(CPU().get_cocci_jobs())
        self.spatch_version = get_spatch_version()
        self.rules_metadata = {}
        self.cve_rules = get_all_cves()

        ipaths = [
            os.path.join('arch', arch, 'include'),
            os.path.join('arch', arch, 'include/generated'),
            os.path.join('arch', arch, 'include/uapi'),
            os.path.join('arch', arch, 'include/generated/uapi'),
            'include',
            'include/uapi',
            'include/generated/uapi'
        ]
        ipaths = map(lambda f: os.path.join(kernel, f), ipaths)
        includes = []
        for i in ipaths:
            includes.append('-I')
            includes.append(i)
        self.includes = includes

        if config:
            parser = KbuildParser(None, arch)
            dirs_to_process = collections.OrderedDict()
            parser.init_class.process(parser, dirs_to_process, kernel)

            for item in dirs_to_process:
                descend = parser.init_class.get_file_for_subdirectory(item)
                parser.process_kbuild_or_makefile(descend, dirs_to_process[item])

            self.config_map = parser.get_config()
            if config != '-':
                self.config_file = config
                self.config = Config(config)
            else:
                self.config_file = None
                self.config = None
        else:
            self.config_file = None
            self.config_map = None
            self.config = None

    def get_grep_pattern(self, rule):
        is_fix = False
        start = False
        patterns = []
        with open(rule, 'rt') as fh:
            for line in fh:
                line = line.strip()
                if line == 'FIX':
                    is_fix = True
                    start = True
                    continue
                elif line == 'ERROR':
                    start = True
                    continue
                if start and line:
                    patterns.append(line)
        return (is_fix, patterns)

    def check_cve(self, cve, all_files=False):
        result = {}
        is_grep = False
        rule = self.cve_rules[cve]
        if rule.endswith('.grep'):
            is_grep = True

        files = []
        if not all_files:
            files = self.get_rule_files(cve)
            files = map(lambda f: os.path.join(self.kernel, f), files)
            files = filter(lambda f: os.path.exists(f), files)
            files = list(files)
        if not files:
            files = [ self.kernel ]

        includes = self.includes.copy()
        kconfig = os.path.join(self.kernel, 'include/linux/kconfig.h')
        if os.path.exists(kconfig):
            includes.append('--include')
            includes.append(kconfig)

        logging.info('Checking: ' + cve)

        output = ''
        run = None
        if not is_grep:
            rule_ver = self.get_rule_version(cve)
            if rule_ver and rule_ver > self.spatch_version:
                raise UnsupportedVersion(self.spatch_version, cve, rule_ver)
            try:
                cocci_cmd = ['spatch', '--no-includes', '--include-headers',
                             '-D', 'detect', '--no-show-diff', '-j', self.cocci_job,
                             *includes,
                             '--chunksize', '1',
                             '--cocci-file', rule, *files]

                logging.debug(' '.join(cocci_cmd))

                run = subprocess.run(cocci_cmd, stdout=PIPE, stderr=PIPE, check=True)
                output = run.stdout.decode('utf-8').strip()
            except subprocess.CalledProcessError as e:
                err = e.stderr.decode('utf-8').split('\n')[-2]
                # Coccinelle 1.0.4 bug workaround
                if ('Sys_error("' + cve + ': No such file or directory")') not in err:
                    raise e
        else:
            (is_fix, patterns) = self.get_grep_pattern(rule)
            args = ['grep', '--include=*.[ch]', '-rPzoe', patterns[0], *files]
            patterns.pop(0)
            run = subprocess.run(args, stdout=PIPE, stderr=PIPE, check=False)
            if run.returncode == 0:
                output = run.stdout
                last = patterns.pop()
                for pattern in patterns:
                    run = subprocess.run(['grep', '-Pzoe', pattern],
                                         input=output, check=False,
                                         stdout=PIPE, stderr=PIPE)
                    if run.returncode != 0:
                        output = ''
                        break
                    output = run.stdout
                if run.returncode == 0:
                    run = subprocess.run(['grep', '-Pzoe', last],
                                         input=output, check=False,
                                         stdout=PIPE, stderr=PIPE)
                    success = run.returncode == 0
                    if is_fix == success:
                        output = ''
                    else:
                        output = 'ERROR'

        if 'ERROR' in output:
            logging.warning('Found: ' + cve)
            if cve in self.metadata:
                info = self.metadata[cve]
                result = info
                if 'cmt_msg' in info:
                    logging.info('MSG: ' + info['cmt_msg'])
                if 'cwe' in info:
                    logging.info('CWE: ' + info['cwe'])
                if 'fix_date' in info:
                    logging.info('FIX DATE: ' + str(datetime.utcfromtimestamp(info['fix_date'])))
            logging.info('https://www.linuxkernelcves.com/cves/' + cve)
            if self.config_map:
                result['config'] = {}
                config_affected = None
                files = {}
                for line in output.split('\n'):
                    file = line.split(':')[0]
                    files[file] = self.config_map.get(file, '')
                if files:
                    logging.info('Affected Files:')
                    for file, config in files.items():
                        result['config'][file] = {}
                        if config:
                            config = simplify_logic(config)
                            result['config'][file]['logic'] = str(config)
                            if self.config:
                                affected = config.subs(self.config.get_mapping())
                                if affected == True:
                                    affected = 'affected'
                                    result['config'][file]['config'] = True
                                    config_affected = True
                                else:
                                    affected = 'not affected'
                                    result['config'][file]['config'] = False
                                    if config_affected == None:
                                        config_affected = False
                                logging.info(' - ' + file + ': ' + str(config) + '\n   ' + self.config_file + ': ' + affected)
                            else:
                                logging.info(' - ' + file + ': ' + str(config))
                        elif not file.endswith('.h'): # TODO: if only .h file, e.g. linux/kernel.h?
                            result['config'][file]['logic'] = True
                            result['config'][file]['config'] = True
                            config_affected = True
                            logging.info(' - ' + file + ': True')
                result['config']['affected'] = config_affected
                if config_affected != None:
                    affected = 'affected'
                    if not config_affected:
                        affected = 'not affected'
                    if self.config:
                        logging.info('Config: ' + self.config_file + ' ' + affected)
                    else:
                        logging.info('Config: any ' + affected)
            result['spatch_output'] = output
            logging.debug(output)
            logging.info('')
            return result
        return False

    def get_rule_metadata(self, cve):
        files = []
        fix = None
        fixes = None
        version = 0

        if cve in self.rules_metadata:
            return self.rules_metadata[cve]

        with open(self.cve_rules[cve], 'rt') as fh:
            for line in fh:
                if not line.startswith('///'):
                    break
                if 'Files:' in line:
                    files = line.partition('Files:')[2].split()
                elif 'Fix:' in line:
                    fix = line.partition('Fix:')[2].strip()
                elif 'Fixes:' in line:
                    fixes = line.partition('Fixes:')[2].strip()
                elif 'Detect-To:' in line:
                    fixes = line.partition('Detect-To:')[2].strip()
                elif 'Version:' in line:
                    version = line.partition('Version:')[2].strip()
                    version = int(version.replace('.', ''))

        meta = {
            'files': files,
            'fix': fix,
            'fixes': fixes,
            'version': version
        }
        self.rules_metadata[cve] = meta
        return meta

    def get_cve_metadata(self, cve):
        return self.metadata.get(cve, None)

    def get_cve_cwe(self, cve):
        return self.metadata[cve].get('cwe', None)

    def get_cves(self):
        return self.cve_rules.keys()

    def get_rule(self, cve):
        return self.cve_rules[cve]

    def get_rule_fix(self, cve):
        return self.get_rule_metadata(cve)['fix']

    def get_rule_fixes(self, cve):
        return self.get_rule_metadata(cve)['fixes']

    def get_rule_files(self, cve):
        return self.get_rule_metadata(cve)['files']

    def get_rule_version(self, cve):
        return self.get_rule_metadata(cve)['version']



import os
import re
import subprocess
import pkg_resources
import json
import gzip
from shutil import which

def dir_path(path):
    if os.path.isdir(path):
        return path
    raise NotADirectoryError(path)

def file_path(path):
    if os.path.isfile(path) or path == '-':
        return path
    raise FileNotFoundError(path)

def tool_exists(name):
    return which(name) is not None

def removesuffix(string, suffix):
    if suffix and string.endswith(suffix):
        return string[:-len(suffix)]
    return string[:]

def get_config_data(path):
    with open(path, 'rt') as fh:
        ver_pattern = re.compile(r'# Linux/(\S+)\s+(\S+)\s+Kernel Configuration')
        for line in fh:
            res = ver_pattern.match(line)
            if res:
                return {'arch': res.group(1), 'version': res.group(2)}
    return {}

def get_kernel_version(path):
    version = {}
    with open(os.path.join(path, 'Makefile'), 'rt') as fh:
        def getparam():
            line = fh.readline()
            if line.startswith('#'):
                line = fh.readline()
            return line.split('=')[1].strip()
        version['version'] = getparam()
        version['patchlevel'] = getparam()
        version['sublevel'] = getparam()
        version['extraversion'] = getparam()
        version['name'] = getparam()
    version['full'] = '.'.join([version['version'], version['patchlevel'], version['sublevel']]) + version['extraversion']
    return version

def get_cvehound_version():
    version = pkg_resources.get_distribution('cvehound').version
    location = pkg_resources.get_distribution('cvehound').location

    if not os.path.exists(os.path.join(location, '.git')):
        return version

    try:
        desc = ['git', 'describe', '--tags', '--dirty']
        version = subprocess.check_output(
            desc, cwd=location, stderr=subprocess.DEVNULL, universal_newlines=True
        ).strip()
    finally:
        return version

def get_spatch_version():
    version = subprocess.check_output(
            ['spatch', '--version'],
            stderr=subprocess.DEVNULL, universal_newlines=True
    ).strip().split('\n')[0]
    res = re.match(r'spatch\s+version\s+([\d.]+)', version)
    return int(res.group(1).replace('.', ''))

def get_all_cves():
    cves = {}
    for cve in pkg_resources.resource_listdir('cvehound', 'cve/'):
        name = removesuffix(removesuffix(cve, '.grep'), '.cocci')
        cves[name] = pkg_resources.resource_filename('cvehound', 'cve/' + cve)
    return cves

def get_cves_metadata():
    cves = pkg_resources.resource_filename('cvehound', 'data/kernel_cves.json.gz')
    data = None
    with gzip.open(cves, 'rt', encoding='utf-8') as fh:
        data = json.load(fh)
    return data



#!/usr/bin/env python3

import pytest
from cvehound.exception import UnsupportedVersion

@pytest.mark.fast
@pytest.mark.notbackported(
    ('branch', 'cve'),
    [
        ('origin/linux-4.4.y', 'CVE-2019-12382'),
        ('origin/linux-4.9.y', 'CVE-2019-12382'),
        ('origin/linux-4.4.y', 'CVE-2019-15924'),
        ('origin/linux-4.4.y', 'CVE-2020-27777'),
        ('origin/linux-4.9.y', 'CVE-2020-27777'),
        ('origin/linux-4.4.y', 'CVE-2020-29569'),
        ('origin/linux-4.19.y', 'CVE-2019-12455'),
        ('origin/linux-4.14.y', 'CVE-2019-12455'),
        ('origin/linux-4.9.y', 'CVE-2019-12455'),
        ('origin/linux-5.4.y', 'CVE-2020-27825'),
        ('origin/linux-4.19.y', 'CVE-2020-27825'),
        ('origin/linux-4.14.y', 'CVE-2020-27825'),
        ('origin/linux-4.9.y', 'CVE-2020-27825'),
        ('origin/linux-4.4.y', 'CVE-2020-27825')
    ]
)
def test_on_branch(hound, repo, branch, cve):
    repo.git.checkout(branch)
    try:
        assert not hound.check_cve(cve), cve + ' on ' + branch
    except UnsupportedVersion:
        pytest.skip('Unsupported spatch version')



#!/usr/bin/env python3

import re
from cvehound.cwe import CWE
from cvehound.util import get_cves_metadata

def test_metadata(hound, cve):
    meta = hound.get_rule_metadata(cve)

    assert 'files' in meta, 'no "Files:" tag in the rule'
    assert 'fix' in meta, 'no "Fix:" tag in the rule'
    assert 'fixes' in meta, 'no "Fixes:" or "Detect-To:" tag in the rule'

    found = False
    cve_id = re.compile(r'CVE-\d{4}-\d{4,7}')
    with open(hound.get_rule(cve), 'rt') as fh:
        for line in fh:
            res = cve_id.search(line)
            if res:
                assert res.group(0) == cve, 'wrong CVE-id in the rule'
                found = True

    assert found, 'no CVE-id in the rule'
    assert hound.get_cve_metadata(cve), 'no metadata in kernel_cves.json'

def test_cwe():
    for cwe in CWE:
        if cwe == 'Other' or cwe == 'Unspecified':
            continue
        assert CWE[cwe], 'No CWE-id for "{}"'.format(cwe)

def test_cves_metadata_cwe(hound):
    meta = hound.metadata
    for cve in meta:
        if 'cwe' in meta[cve]:
            assert meta[cve]['cwe'] in CWE, 'Unknown CWE description "{}"'.format(meta[cve]['cwe'])



#!/usr/bin/env python3

import sys
import pkg_resources
import gzip
import urllib.request
import json
import subprocess
from datetime import datetime

def main(args=sys.argv[1:]):
    filename = pkg_resources.resource_filename('cvehound', 'data/kernel_cves.json.gz')

    with urllib.request.urlopen('https://github.com/nluedtke/linux_kernel_cves/raw/master/data/kernel_cves.json') as fh:
        js = json.loads(fh.read().decode('utf-8'))

    for cve, info in js.items():
        fix = info.get('fixes', '')
        if fix:
            try:
                info['fix_date'] = int(subprocess.check_output(
                    ['git', 'show', '-s', '--format=%ct', fix], cwd=args[0], stderr=subprocess.DEVNULL, universal_newlines=True
                ).strip())
            except Exception:
                pass

    with gzip.open(filename, 'wt', encoding='utf-8') as fh:
        json.dump(js, fh)

if __name__ == '__main__':
    main(sys.argv[1:])
